package com.cht.tas.ivr;

import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.PreDestroy;
import java.nio.charset.StandardCharsets;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * PhoneConfig 內容
 * {
 *   "serviceNumber": "0281920613",
 *   "ivrData": [
 *     {
 *       "node": "MAIN",
 *       "text": "歡迎使用中華電信互動式選單",
 *       "nextNode": "menu"
 *     },
 *     {
 *       "node": "menu",
 *       "text": "查詢天氣請按 1，查詢台北空氣品質請按2",
 *       "collectDTMF": true,
 *       "DTMF":{"1" : "weather_menu", "2": "CUSTOM"}
 *     },
 *                {
 * 			"node": "weather_menu",
 *       "text": "台北天氣請按1,台中天氣請按2,高雄天氣請按3,結束請按任意鍵",
 *       "collectDTMF": "true"
 *
 *        },
 *        {
 * 			"node":"exit",
 * 			"text":"再見",
 * 			"nextNode":"END"
 *        }
 *   ]
 * }
 *
 * */
@Component
public class IVRMqtt {
    private final Logger Log = LoggerFactory.getLogger(IVRMqtt.class);

    @Value("${mqtt.phone-conn.eventTopic}")
    private String eventTopic = "";

    @Value("${mqtt.phone-conn.actionDebugTopic}")
    private String actionDebugTopic = "";

    @Autowired
    private MqttTas mqttTas;

    @Autowired
    private IVREvent IVREvent;

    private ExecutorService publishService = Executors.newSingleThreadExecutor();


    public void setSubscribeAllTopic() {
        setSubscribeActionDebugTopic();
        setSubscribeEventTopic();
    }

    //若 IVR 系統發生錯誤，則印出除錯訊息
    public void setSubscribeActionDebugTopic(){
        Log.debug("topic:{}", actionDebugTopic);
        mqttTas.setSubscribeTopicResp(actionDebugTopic,((topic, message) -> {
            Log.debug("messageArrived topic: " +  topic);

            String msg = new String(message.getPayload(), StandardCharsets.UTF_8);
            Log.debug("DEBUG RECV:" + msg);
        }));
    }

    //設定互動式選單的內容
    public void setSubscribeEventTopic(){
        Log.debug("topic:{}", eventTopic);
        mqttTas.setSubscribeTopicResp(eventTopic, (topic, message) -> {
            Log.debug("messageArrived topic: " +  topic);

            String msg = new String(message.getPayload(), StandardCharsets.UTF_8);
            JSONObject reqJObjs = new JSONObject(msg);
            String type = reqJObjs.getString("type");

            //收到 MQTT Call-in IVR 互動選單的 request
            if ("request".equals(type)) {

                //拆解 node 節點的內容及參數
                String[] mArgs = reqJObjs.getString("node").split(",");

                //節點內容
                String node = mArgs[0];

                //節點參數
                String param = "";
                if (mArgs.length > 1) {
                    param = mArgs[1];
                }

                String finalParam = param;
                publishService.submit(()-> {
                    switch (node) {
                        case "menu":{
                            //用戶在 menu 時按下 2
                            if (finalParam.equals("2")){
                                IVREvent.menu_2(reqJObjs);
                            }
                            break;
                        }
                        case "weather_menu":{
                            //用戶在 weather_menu 按下按鍵
                            if (finalParam.equals("1")) {
                                IVREvent.weather_menu_1(reqJObjs);
                            } else if (finalParam.equals("2")) {
                                IVREvent.weather_menu_2(reqJObjs);
                            } else if (finalParam.equals("3")) {
                                IVREvent.weather_menu_3(reqJObjs);
                            } else {
                                IVREvent.weather_menu_other(reqJObjs);
                            }
                            break;
                        }
                        default:{
                            Log.debug("default:{}", node);
                        }
                    }
                });
            }
        });
    }

    @PreDestroy
    private void destory(){
        publishService.shutdown();
    }
}
